import 'dart:cli';
import 'dart:convert';
import 'dart:io';
import 'dart:async';

import 'package:file_utils/file_utils.dart';

import 'package:dartcli/stack_list.dart';

StackList<Directory> directoryStack = StackList();

void mv(String from, String to) {
  File(from).renameSync(to);
}

String get pwd {
  return Directory.current.path;
}

void echo(String text) {
  print(text);
}

void cd(String path) {
  Directory.current = path;
}

List<String> get fileList {
  List<String> files = List();
  Directory.current.listSync().forEach((file) => files.add(file.path));
  return files;
}

///
/// Returns the list of files in the cwd that
/// match the passed glob pattern.
///
List<String> wildcard(String pattern, {bool caseSensitive}) {
  List<String> files = List();

  FileUtils.glob(pattern,
      caseSensitive: caseSensitive, notify: (file) => files.add(file));

  return files;
}

void touch(String path, {bool create = true}) {
  FileUtils.touch([path], create: true);
}

////
/// Push the pwd onto the stack and change the
/// current directory to [path].
void push(String path) {
  directoryStack.push(Directory.current);
  Directory.current = path;
}

///
/// Change the working directory back
/// to its location before [push] was called.
void pop() {
  Directory.current = directoryStack.pop();
}

void mkdir(String path, {bool createPath}) {
  Directory(path).createSync(recursive: createPath);
}

bool isFile(String path) {
  FileSystemEntityType fromType = FileSystemEntity.typeSync(path);
  return (fromType == FileSystemEntityType.file);
}

/// true if the given path is a directory.
bool isDirectory(String path) {
  FileSystemEntityType fromType = FileSystemEntity.typeSync(path);
  return (fromType == FileSystemEntityType.directory);
}

/// checks if the given [path] exists.
bool exists(String path) {
  return File(path).existsSync();
}

///
/// Reads user input from stdin and returns it as a string.
String read({String prompt}) {
  if (prompt != null) {
    print(prompt);
  }
  var line = stdin.readLineSync(encoding: Encoding.getByName('utf-8'));
  return line;
}

/// Prints the contents of the give file to stdout.
///
void cat(String path) {
  File sourceFile = File(path);

  waitFor<void>(sourceFile
      .openRead()
      .transform(utf8.decoder)
      .transform(LineSplitter())
      .forEach((line) => print(line)));
}

void rm(String path, {bool ask}) {
  bool remove = true;
  if (ask) {
    remove = false;
    var yes = read(prompt: "rm: remove regular file '${path}'? y/N");
    if (yes == "y") {
      remove = true;
    }

    if (remove == true) {
      File(path).delete();
    }
  }
}

enum Interval { seconds, millseconds, minutes }

void sleep(int duration, {Interval interval = Interval.seconds}) {
  Duration _duration;
  switch (interval) {
    case Interval.seconds:
      _duration = Duration(seconds: duration);
      break;
    case Interval.millseconds:
      _duration = Duration(microseconds: duration);
      break;
    case Interval.minutes:
      _duration = Duration(minutes: duration);
      break;
  }

  waitFor<void>(Future.delayed(_duration));
}
